//+------------------------------------------------------------------+
//|                                   Heiken_Ashi_SmoothedSignal.mqh |
//|                             Copyright  2011,   Nikolay Kositsin | 
//|                              Khabarovsk,   farria@mail.redcom.ru | 
//+------------------------------------------------------------------+
#property copyright "Copyright  2011, Nikolay Kositsin"
#property link      "farria@mail.redcom.ru"
//+------------------------------------------------------------------+
//|                                                   |
//+------------------------------------------------------------------+
#property tester_indicator "Heiken_Ashi_Smoothed.ex5"
#include <Expert\ExpertSignal.mqh>
// wizard description start
//+------------------------------------------------------------------+ 
//|                                                |
//+------------------------------------------------------------------+ 
#define OPEN_LONG     80  //          
#define OPEN_SHORT    80  //          
#define CLOSE_LONG    40  //          
#define CLOSE_SHORT   40  //          
#define REVERSE_LONG  100 //         
#define REVERSE_SHORT 100 //         
#define NO_SIGNAL      0  //         
//+------------------------------------------------------------------+
//| Description of the class                                         |
//| Title=,    Heiken_Ashi_Smoothed     |
//| Type=SignalAdvanced                                              |
//| Name=Heiken_Ashi_Sm                                              |
//| Class=CHeiken_Ashi_SmoothedSignal                                |
//| Page=                                                            |
//| Parameter=BuyPosOpen,bool,true,           |
//| Parameter=SellPosOpen,bool,true,          |
//| Parameter=BuyPosClose,bool,true,      |
//| Parameter=SellPosClose,bool,true,     |
//| Parameter=Ind_Timeframe,ENUM_TIMEFRAMES,PERIOD_H4,      |
//| Parameter=MA_SMethod,uint,5,  (1 - 10)            |
//| Parameter=MA_Length,uint,30,                    |
//| Parameter=MA_Phase,uint,100,                   |
//| Parameter=SignalBar,uint,1,              |
//+------------------------------------------------------------------+
// wizard description end
//+------------------------------------------------------------------+
//|  CHeiken_Ashi_SmoothedSignal.                               |
//| :          |
//| Heiken_Ashi_Smoothed http://www.mql5.com/ru/code/481/.           |
//|                 CExpertSignal.        |
//+------------------------------------------------------------------+
class CHeiken_Ashi_SmoothedSignal : public CExpertSignal
  {
protected:
   CiCustom          m_indicator;        //      Heiken_Ashi_Smoothed

   //---  
   bool              m_BuyPosOpen;       //     
   bool              m_SellPosOpen;      //     
   bool              m_BuyPosClose;      //     
   bool              m_SellPosClose;     //     
   ENUM_TIMEFRAMES   m_Ind_Timeframe;    //   Heiken_Ashi_Smoothed
   uint              m_MA_SMethod;       //   (1 - 10)
   uint              m_MA_Length;        //  
   uint              m_MA_Phase;         //  
   uint              m_SignalBar;        //       

public:
                     CHeiken_Ashi_SmoothedSignal();

   //---    
   void              BuyPosOpen(bool value)                  { m_BuyPosOpen=value;       }
   void              SellPosOpen(bool value)                 { m_SellPosOpen=value;      }
   void              BuyPosClose(bool value)                 { m_BuyPosClose=value;      }
   void              SellPosClose(bool value)                { m_SellPosClose=value;     }
   void              Ind_Timeframe(ENUM_TIMEFRAMES value)    { m_Ind_Timeframe=value;    }
   void              MA_SMethod(uint value)                  { m_MA_SMethod=value;       }
   void              MA_Length(uint value)                   { m_MA_Length=value;        }
   void              MA_Phase(uint value)                    { m_MA_Phase=value;         }
   void              SignalBar(uint value)                   { m_SignalBar=value;        }

   //---    
   virtual bool      ValidationSettings();
   //---    
   virtual bool      InitIndicators(CIndicators *indicators); //  
   //---      
   virtual int       LongCondition();
   virtual int       ShortCondition();

   bool              InitHeiken_Ashi_Sm(CIndicators *indicators);  // ,   Heiken_Ashi_Smoothed

  };
//+------------------------------------------------------------------+
//|  CHeiken_Ashi_SmoothedSignal.                         |
//| INPUT:  .                                                     |
//| OUTPUT: .                                                     |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
void CHeiken_Ashi_SmoothedSignal::CHeiken_Ashi_SmoothedSignal()
  {
//---   -
   m_BuyPosOpen=true;
   m_SellPosOpen=true;
   m_BuyPosClose=true;
   m_SellPosClose=true;
   m_Ind_Timeframe=PERIOD_H4;
   m_MA_SMethod=5;
   m_MA_Length=30;
   m_MA_Phase=100;
   m_SignalBar=1;
   m_used_series=USE_SERIES_OPEN+USE_SERIES_HIGH+USE_SERIES_LOW+USE_SERIES_CLOSE;
  }
//+------------------------------------------------------------------+
//|   .                                   |
//| INPUT:  .                                                     |
//| OUTPUT: true-  ,  false.             |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CHeiken_Ashi_SmoothedSignal::ValidationSettings()
  {
//---  
   if(m_MA_SMethod<1)
     {
      printf(__FUNCTION__+":        1!");
      printf(__FUNCTION__+":       1!");
      m_MA_SMethod=1;
     }

   if(m_MA_SMethod>10)
     {
      printf(__FUNCTION__+":        10!");
      printf(__FUNCTION__+":       10!");
      m_MA_SMethod=10;
     }

//---  
   return(true);
  }
//+------------------------------------------------------------------+
//|    .                           |
//| INPUT:  indicators -   -               |
//|                        .                    |
//| OUTPUT: true-   ,  false.         |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CHeiken_Ashi_SmoothedSignal::InitIndicators(CIndicators *indicators)
  {
//---  
   if(indicators==NULL) return(false);

//---  
   if(!InitHeiken_Ashi_Sm(indicators)) return(false);

//---  
   return(true);
  }
//+------------------------------------------------------------------+
//|   Heiken_Ashi_Smoothed.                   |
//| INPUT:  indicators -   -               |
//|                        .                    |
//| OUTPUT: true-   ,  false.         |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CHeiken_Ashi_SmoothedSignal::InitHeiken_Ashi_Sm(CIndicators *indicators)
  {
//---  
   if(indicators==NULL) return(false);

//---    
   if(!indicators.Add(GetPointer(m_indicator)))
     {
      printf(__FUNCTION__+":   ");
      return(false);
     }

//---   
   MqlParam parameters[4];

   parameters[0].type=TYPE_STRING;
   parameters[0].string_value="Heiken_Ashi_Smoothed.ex5";

   parameters[1].type=TYPE_INT;
   parameters[1].integer_value=m_MA_SMethod;

   parameters[2].type=TYPE_INT;
   parameters[2].integer_value=m_MA_Length;

   parameters[3].type=TYPE_INT;
   parameters[3].integer_value=m_MA_Phase;

//---     
   if(!m_indicator.Create(m_symbol.Name(),m_Ind_Timeframe,IND_CUSTOM,4,parameters))
     {
      printf(__FUNCTION__+":   ");
      return(false);
     }

//---  
   if(!m_indicator.NumBuffers(5)) return(false);

//---     Heiken_Ashi_Smoothed
   return(true);
  }
//+------------------------------------------------------------------+
//|                              |
//|                            |
//| INPUT:                                                        |
//| OUTPUT:    0  100                                   |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
int CHeiken_Ashi_SmoothedSignal::LongCondition()
  {
//---       4  Heiken_Ashi_Smoothed
   double Signal0=m_indicator.GetData(4,m_SignalBar);
   double Signal1=m_indicator.GetData(4,m_SignalBar+1);

//---    
   if(Signal0==0)
     {
      if(Signal1==1)
        {
         if(m_BuyPosOpen)
           {
            if(m_SellPosClose) return(REVERSE_SHORT);
            else return(OPEN_LONG);
           }
         if(m_SellPosClose) return(CLOSE_SHORT);
        }
     }
//---   
   return(NO_SIGNAL);
  }
//+------------------------------------------------------------------+
//|                              |
//|                            |
//| INPUT:                                                        |
//| OUTPUT:    0  100                                   |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
int CHeiken_Ashi_SmoothedSignal::ShortCondition()
  {
//---       4  Heiken_Ashi_Smoothed
   double Signal0=m_indicator.GetData(4,m_SignalBar);
   double Signal1=m_indicator.GetData(4,m_SignalBar+1);

//---    
   if(Signal0==1)
     {
      if(Signal1==0)
        {
         if(m_SellPosOpen)
           {
            if(m_BuyPosClose) return(REVERSE_LONG);
            else return(OPEN_LONG);
           }
         if(m_BuyPosClose) return(CLOSE_LONG);
        }
     }
//---      
   return(NO_SIGNAL);
  }
//+------------------------------------------------------------------+
